@@ -1,30 +1,29 @@
 /*
- * Sun RPC is a product of Sun Microsystems, Inc. and is provided for
- * unrestricted use provided that this legend is included on all tape
- * media and as a part of the software program in whole or part.  Users
- * may copy or modify Sun RPC without charge, but are not authorized
- * to license or distribute it to anyone else except as part of a product or
- * program developed by the user.
- * 
- * SUN RPC IS PROVIDED AS IS WITH NO WARRANTIES OF ANY KIND INCLUDING THE
- * WARRANTIES OF DESIGN, MERCHANTIBILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE, OR ARISING FROM A COURSE OF DEALING, USAGE OR TRADE PRACTICE.
- * 
- * Sun RPC is provided with no support and without any obligation on the
- * part of Sun Microsystems, Inc. to assist in its use, correction,
- * modification or enhancement.
- * 
- * SUN MICROSYSTEMS, INC. SHALL HAVE NO LIABILITY WITH RESPECT TO THE
- * INFRINGEMENT OF COPYRIGHTS, TRADE SECRETS OR ANY PATENTS BY SUN RPC
- * OR ANY PART THEREOF.
- * 
- * In no event will Sun Microsystems, Inc. be liable for any lost revenue
- * or profits or other special, indirect and consequential damages, even if
- * Sun has been advised of the possibility of such damages.
- * 
- * Sun Microsystems, Inc.
- * 2550 Garcia Avenue
- * Mountain View, California  94043
+ * Copyright (c) 2009, Sun Microsystems, Inc.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
+ * - Redistributions of source code must retain the above copyright notice,
+ *   this list of conditions and the following disclaimer.
+ * - Redistributions in binary form must reproduce the above copyright notice,
+ *   this list of conditions and the following disclaimer in the documentation
+ *   and/or other materials provided with the distribution.
+ * - Neither the name of Sun Microsystems, Inc. nor the names of its
+ *   contributors may be used to endorse or promote products derived
+ *   from this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
  */
 /*
  * Copyright (c) 1986-1991 by Sun Microsystems Inc. 
@@ -226,7 +225,7 @@ __rpc_getconfip(nettype)
 	char *netid_tcp = (char *) NULL;
 	char *netid_udp = (char *) NULL;
 	struct netconfig *dummy;
-	static thread_key_t tcp_key = -1, udp_key = -1;
+	extern thread_key_t tcp_key, udp_key;
 	extern mutex_t tsd_lock;
 
 	if (tcp_key == -1) {
@@ -813,6 +812,11 @@ int
 __rpc_sockisbound(int fd)
 {
 	struct sockaddr_storage ss;
+	union {
+		struct sockaddr_in  sin;
+		struct sockaddr_in6 sin6;
+		struct sockaddr_un  usin;
+	} u_addr;
 	socklen_t slen;
 
 	slen = sizeof (struct sockaddr_storage);
@@ -821,17 +825,17 @@ __rpc_sockisbound(int fd)
 
 	switch (ss.ss_family) {
 		case AF_INET:
-			return (((struct sockaddr_in *)
-			    (void *)&ss)->sin_port != 0);
+			memcpy(&u_addr.sin, &ss, sizeof(u_addr.sin)); 
+			return (u_addr.sin.sin_port != 0);
 #ifdef INET6
 		case AF_INET6:
-			return (((struct sockaddr_in6 *)
-			    (void *)&ss)->sin6_port != 0);
+			memcpy(&u_addr.sin6, &ss, sizeof(u_addr.sin6)); 
+			return (u_addr.sin6.sin6_port != 0);
 #endif
 		case AF_LOCAL:
 			/* XXX check this */
-			return (((struct sockaddr_un *)
-			    (void *)&ss)->sun_path[0] != '\0');
+			memcpy(&u_addr.usin, &ss, sizeof(u_addr.usin)); 
+			return (u_addr.usin.sun_path[0] != 0);
 		default:
 			break;
 	}
